/* RC-Switch;

(c) 2007 by Prikupets Andrey

*/

/*

What a servo o/p look like:

                <-------------- 21 ms -------------------------->               

                   <- 1 to 2.5ms ->
                -----------------                               ----
channel x     --                 -------------------------------

  typical values

                Mine,         Colins   worst case   Used ( 1 lot of 35 towards MIDDLE)
 full trim       990 (est)    1010      n/a
left / up       1120          1090      1120        1155
MIDDLE          1520          1500      1500        1500
right / down    1980          1950      1950        1915
 full trim      2110 (est)    2070      n/a

  for simple switch operation

  if < 1300 => left
  if > 1700 => right
  otherwise MIDDLE

*/

//#define PIC12F629
#define PIC12F675
//#define PIC12C509

#ifdef PIC12F629
#include "12F629.H" 
#else
#ifdef PIC12F675
#include "12F675.H" 
#else
#include "12C509A.H"
#endif
#endif

/*
bit 0  0}
bit 1  1}=> Internal RC
bit 2  0=>Watchdog disabled
bit 3  1=>No code protecr
bit 4  0=>mclr pin NOT used
*/

#if defined PIC12F629 || defined PIC12F675

#define NO_MCLR     &= ~0x80
#define CP_off      |= 0x2180
#pragma config  BODEN=off, NO_MCLR, PWRTE=on, FOSC=4, WDTE=off, CP_off

#else

#define NO_MCLR     &= ~0x80
#define CP_off      |= 0xFFE8
#define INTERNAL_RX |= 2
#pragma config  INTERNAL_RX, CP_off, WDTE=off

#endif

/***********/
/* DEFINES */
/***********/

#define PRESCALER_VALUE   3     //A value of 3 gives a pre-scaler of 16, this meas the counter will count to 4096S

#define MICRO_SECONDS_AT_FULL_COUNT 4096
#define MICRO_SECONDS_PER_COUNT     (4096/256)

#define STICK_LEFT_TIME   1300
#define STICK_RIGHT_TIME  1700

#define STICK_LEFT_COUNT  (STICK_LEFT_TIME / MICRO_SECONDS_PER_COUNT)   // 81
#define STICK_RIGHT_COUNT  (STICK_RIGHT_TIME / MICRO_SECONDS_PER_COUNT)  // 106

#define FALSE                   0
#define TRUE                    1


/************/
/* VARIABLES */
/************/

/* Outputs */

// Assignment for PIC
#pragma bit OP_NO_REV	@ GPIO.5
#pragma bit OP_PROJ	@ GPIO.0
#pragma bit OP_LED	@ GPIO.1

/* Input */
// NB bit 3 must be an input
#pragma bit ip_rx @ GPIO.3  // bit stream from receiver


// General counter
unsigned char nTimer;

#define STABLE_COUNT 5  
unsigned char nStabCount;

/************************************************************/
// Routine to read the input and measure how long it is high
/************************************************************/
static void ReadInput()
{
      // Initally look a low
      while (ip_rx)
      {
      }

      // Now wait for it to go high
      while (!ip_rx)
      {
      }

      // measure how long it is high
      TMR0 = 0;
      while (ip_rx)
      {
      }

      // and return the value
      nTimer = TMR0;

} // static char ReadInput()



void Delay100us (unsigned char aTs) // pause for aTs*100 us;
{ // 4+x*3 us = 4+3*32=100 us;
 do  {
     char i = 33;            /* 2 IS */
     do ; while ( --i > 0);  /* 26 * 3 - 1 = 77 IS */
 } while ( --aTs > 0);         /* 3 IS */
}

void DelayMs (unsigned char aTs) // pause for aTs*1 ms;
{
 for (; aTs > 0; aTs--) {
   clrwdt();
   Delay100us (10);
 }
}

void Beep (unsigned char pDuration, unsigned char pPause) {
 OP_LED = TRUE;
 DelayMs (pDuration);
 OP_LED = FALSE;
 if (pPause)
   DelayMs (pPause);
}

/***************************************************************/
/* MAIN                                                        */
/***************************************************************/

char GetOsccal();							// Defined at end of file

void main( void)
{
#if defined PIC12F629 || defined PIC12F675
  W = GetOsccal();						// Set built-in oscillator calibration
#endif
  OSCCAL = W;

  GPIO = 0;

#if defined PIC12F629 || defined PIC12F675
  CMCON = 0b.0000.0111; // comparator is turned off;
  WPU = 0b00100000; // pull-up for button (GP5) jumper;
  TRISIO = 0b00101000;        /* bit 3 i/p bits 0,1,2,4,5 o/p */
#ifdef PIC12F675
  ANSEL = 0;
  ADIE = 0;
  ADCON0 = 0b10.0.000.00;     //  Turn off the ADC
#endif
#else
  TRIS = 0b00101000;        /* bit 3 i/p bits 0,1,2,4,5 o/p */
#endif

  OPTION = PRESCALER_VALUE;         /* prescale timer by 16, pull-ups enabled */

  TMR0 = 0;

  nTimer = 0;

  // wait for everything to settle!!!  
  unsigned char i = 0;
  for (;i < 3; i++) 
    Beep(100,100);

  nStabCount = 0;

  // Main loop
  while (1)
  {

    // Read current value of servo input
    ReadInput();

    if (nTimer > STICK_RIGHT_COUNT)
    {
      // stick is right
      if (nStabCount >= STABLE_COUNT) {	  
        OP_LED = OP_PROJ = OP_NO_REV; // usually, [1]; 
      } else {
        nStabCount++;
      }
    }
    else
    {
      // stick is in the middle or left
      nStabCount = 0;
      OP_LED = OP_PROJ = !OP_NO_REV; // usually, [0]; 
    }
  } // while (1)

} // main


#if defined PIC12F629 || defined PIC12F675
#pragma origin 0x3FF
// This is a dummy - actual procedure is preset in chip and will override
// A change of 1 causes about 1% change in frequency
char GetOsccal() {return 0x0;}
#endif

